# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from StringIO import StringIO
import urllib2

from mygpoclient import http
from mygpoclient import json

import unittest
import minimock

class Test_JsonClient(unittest.TestCase):
    USERNAME = 'john'
    PASSWORD = 'secret'

    def setUp(self):
        self.mockopener = minimock.Mock('urllib2.OpenerDirector')
        urllib2.build_opener = minimock.Mock('urllib2.build_opener')
        urllib2.build_opener.mock_returns = self.mockopener

    def tearDown(self):
        minimock.restore()

    def mock_setHttpResponse(self, value):
        self.mockopener.open.mock_returns = StringIO(value)

    def test_parseResponse_worksWithDictionary(self):
        client = json.JsonClient(self.USERNAME, self.PASSWORD)
        self.mock_setHttpResponse('{"a": "B", "c": "D"}')
        items = list(sorted(client.GET('/').items()))
        self.assertEquals(items, [('a', 'B'), ('c', 'D')])

    def test_parseResponse_worksWithIntegerList(self):
        client = json.JsonClient(self.USERNAME, self.PASSWORD)
        self.mock_setHttpResponse('[1,2,3,6,7]')
        self.assertEquals(client.GET('/'), [1,2,3,6,7])

    def test_parseResponse_emptyString_returnsNone(self):
        client = json.JsonClient(self.USERNAME, self.PASSWORD)
        self.mock_setHttpResponse('')
        self.assertEquals(client.GET('/'), None)

    def test_invalidContent_raisesJsonException(self):
        client = json.JsonClient(self.USERNAME, self.PASSWORD)
        self.mock_setHttpResponse('this is not a valid json string')
        self.assertRaises(json.JsonException, client.GET, '/')





# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""gpodder.net API Client Library

This is mygpoclient, the gpodder.net API Client Library for Python.
"""

__author__ = 'Thomas Perl <thp@gpodder.org>'
__version__ = '1.7'
__website__ = 'http://thp.io/2010/mygpoclient/'
__license__ = 'GNU General Public License v3 or later'

# Default settings for the API client (server hostname and API version)
HOST = 'gpodder.net'
VERSION = 2
TOPLIST_DEFAULT = 50

# You can overwrite this value from your application if you want
user_agent = 'mygpoclient/%s (+%s)' % (__version__, __website__)

# Version checking
def require_version(minimum_required):
    """Require a minimum version of the library

    Returns True if the minimum library version constraint is
    satisfied, False otherwise. Use this to check for newer API
    methods and changes in the public API as described in NEWS.

    >>> require_version('1.0')
    True
    >>> require_version('1.2')
    True
    >>> require_version(__version__)
    True
    >>> require_version('99.99')
    False
    """
    this_version = [int(x) for x in __version__.split('.')]
    minimum_required = [int(x) for x in minimum_required.split('.')]
    return minimum_required <= this_version




# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mygpoclient import http

import unittest
import multiprocessing
import BaseHTTPServer

def http_server(port, username, password, response):
    storage = {}
    class Handler(BaseHTTPServer.BaseHTTPRequestHandler):
        def __init__(self, *args, **kwargs):
            BaseHTTPServer.BaseHTTPRequestHandler.__init__(self, *args, **kwargs)

        def _checks(self):
            if not self._check_auth():
                return False
            elif not self._check_errors():
                return False
            else:
                return True

        def _check_auth(self):
            if self.path.startswith('/auth'):
                authorization = self.headers.get('authorization', None)
                if authorization is not None:
                    auth_type, credentials = authorization.split(None, 1)
                    if auth_type.lower() == 'basic':
                        auth_user, auth_pass = credentials.decode('base64').split(':', 1)
                        if username == auth_user and password == auth_pass:
                            return True

                self.send_response(401)
                self.send_header('WWW-Authenticate', 'Basic realm="Fake HTTP Server"')
                self.end_headers()
                self.wfile.close()
                return False

            return True

        def _check_errors(self):
            if self.path.startswith('/badrequest'):
                self.send_response(400)
                self.end_headers()
                self.wfile.close()
                return False
            elif self.path.startswith('/notfound'):
                self.send_response(404)
                self.end_headers()
                self.wfile.close()
                return False
            elif self.path.startswith('/invaliderror'):
                self.send_response(444)
                self.end_headers()
                self.wfile.close()
                return False

            return True

        def do_POST(self):
            if not self._checks():
                return

            input_data = self.rfile.read(int(self.headers.get('content-length')))
            self.send_response(200)
            self.end_headers()
            self.wfile.write(input_data.encode('rot13'))
            self.wfile.close()

        def do_PUT(self):
            if not self._checks():
                return

            input_data = self.rfile.read(int(self.headers.get('content-length')))
            storage[self.path] = input_data
            self.send_response(200)
            self.end_headers()
            self.wfile.write('PUT OK')
            self.wfile.close()

        def do_GET(self):
            if not self._checks():
                return

            self.send_response(200)
            self.end_headers()
            if self.path in storage:
                self.wfile.write(storage[self.path])
            else:
                self.wfile.write(response)
            self.wfile.close()

        def log_request(*args):
            pass

    BaseHTTPServer.HTTPServer(('127.0.0.1', port), Handler).serve_forever()

class Test_HttpClient(unittest.TestCase):
    USERNAME = 'john'
    PASSWORD = 'secret'
    PORT = 9876
    URI_BASE = 'http://localhost:%(PORT)d' % locals()
    RESPONSE = 'Test_GET-HTTP-Response-Content'
    DUMMYDATA = 'fq28cnyp3ya8ltcy;ny2t8ay;iweuycowtc'

    def setUp(self):
        self.server_process = multiprocessing.Process(target=http_server,
                args=(self.PORT, self.USERNAME, self.PASSWORD, self.RESPONSE))
        self.server_process.start()
        import time
        time.sleep(.1)

    def tearDown(self):
        self.server_process.terminate()
        import time
        time.sleep(.1)

    def test_UnknownResponse(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/invaliderror'
        self.assertRaises(http.UnknownResponse, client.GET, path)

    def test_NotFound(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/notfound'
        self.assertRaises(http.NotFound, client.GET, path)

    def test_Unauthorized(self):
        client = http.HttpClient('invalid-username', 'invalid-password')
        path = self.URI_BASE+'/auth'
        self.assertRaises(http.Unauthorized, client.GET, path)

    def test_BadRequest(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/badrequest'
        self.assertRaises(http.BadRequest, client.GET, path)

    def test_GET(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/noauth'
        self.assertEquals(client.GET(path), self.RESPONSE)

    def test_authenticated_GET(self):
        client = http.HttpClient(self.USERNAME, self.PASSWORD)
        path = self.URI_BASE+'/auth'
        self.assertEquals(client.GET(path), self.RESPONSE)

    def test_unauthenticated_GET(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/auth'
        self.assertRaises(http.Unauthorized, client.GET, path)

    def test_POST(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/noauth'
        self.assertEquals(client.POST(path, self.DUMMYDATA), self.DUMMYDATA.encode('rot13'))

    def test_authenticated_POST(self):
        client = http.HttpClient(self.USERNAME, self.PASSWORD)
        path = self.URI_BASE+'/auth'
        self.assertEquals(client.POST(path, self.DUMMYDATA), self.DUMMYDATA.encode('rot13'))

    def test_unauthenticated_POST(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/auth'
        self.assertRaises(http.Unauthorized, client.POST, path, self.DUMMYDATA)

    def test_PUT(self):
        client = http.HttpClient()
        path = self.URI_BASE+'/noauth'
        self.assertEquals(client.PUT(path, self.DUMMYDATA), 'PUT OK')

    def test_GET_after_PUT(self):
        client = http.HttpClient()
        for i in range(10):
            path = self.URI_BASE + '/file.%(i)d.txt' % locals()
            client.PUT(path, self.RESPONSE + str(i))
            self.assertEquals(client.GET(path), self.RESPONSE + str(i))





# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Fix gPodder bug 900 (so "import json" doesn't import this module)
from __future__ import absolute_import

import mygpoclient

try:
    # Prefer the usage of the simplejson module, as it
    # is most likely newer if it's installed as module
    # than the built-in json module (and is mandatory
    # in Python versions before 2.6, anyway).
    import simplejson as json
except ImportError:
    # Python 2.6 ships the "json" module by default
    import json

from mygpoclient import http

# Additional exceptions for JSON-related errors
class JsonException(Exception): pass

class JsonClient(http.HttpClient):
    """A HttpClient with built-in JSON support

    This client will automatically marshal and unmarshal data for
    JSON-related web services so that code using this class will
    not need to care about (de-)serialization of data structures.
    """
    def __init__(self, username=None, password=None):
        http.HttpClient.__init__(self, username, password)

    @staticmethod
    def encode(data):
        """Encodes a object into its JSON string repesentation

        >>> JsonClient.encode(None)
        ''
        >>> JsonClient.encode([1,2,3])
        '[1, 2, 3]'
        >>> JsonClient.encode(42)
        '42'
        """
        if data is None:
            return ''
        else:
            return json.dumps(data)

    @staticmethod
    def decode(data):
        """Decodes a response string to a Python object

        >>> JsonClient.decode('')
        >>> JsonClient.decode('[1,2,3]')
        [1, 2, 3]
        >>> JsonClient.decode('42')
        42
        """
        if data == '':
            return None
        else:
            try:
                return json.loads(data)
            except ValueError, ve:
                raise JsonException('Value error while parsing response: ' + data)

    @staticmethod
    def _prepare_request(method, uri, data):
        data = JsonClient.encode(data)
        return http.HttpClient._prepare_request(method, uri, data)

    @staticmethod
    def _process_response(response):
        data = http.HttpClient._process_response(response)
        return JsonClient.decode(data)



# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mygpoclient

import os
import urllib

from mygpoclient import util

class Locator(object):
    """URI Locator for API endpoints

    This helper class abstracts the URIs for the gpodder.net
    webservice and provides a nice facility for generating API
    URIs and checking parameters.
    """
    SIMPLE_FORMATS = ('opml', 'json', 'txt')

    SETTINGS_TYPES = ('account', 'device', 'podcast', 'episode')

    def __init__(self, username, host=mygpoclient.HOST,
            version=mygpoclient.VERSION):
        self._username = username
        self._simple_base = 'http://%(host)s' % locals()
        self._base = 'http://%(host)s/api/%(version)s' % locals()

    def _convert_since(self, since):
        """Convert "since" into a numeric value

        This is internally used for value-checking.
        """
        try:
            return int(since)
        except ValueError:
            raise ValueError('since must be a numeric value (or None)')

    def subscriptions_uri(self, device_id, format='opml'):
        """Get the Simple API URI for a subscription list

        >>> locator = Locator('john')
        >>> locator.subscriptions_uri('n800')
        'http://gpodder.net/subscriptions/john/n800.opml'
        >>> locator.subscriptions_uri('ipod', 'txt')
        'http://gpodder.net/subscriptions/john/ipod.txt'
        """
        if format not in self.SIMPLE_FORMATS:
            raise ValueError('Unsupported file format')

        filename = '%(device_id)s.%(format)s' % locals()
        return util.join(self._simple_base,
                'subscriptions', self._username, filename)

    def toplist_uri(self, count=50, format='opml'):
        """Get the Simple API URI for the toplist

        >>> locator = Locator(None)
        >>> locator.toplist_uri()
        'http://gpodder.net/toplist/50.opml'
        >>> locator.toplist_uri(70)
        'http://gpodder.net/toplist/70.opml'
        >>> locator.toplist_uri(10, 'json')
        'http://gpodder.net/toplist/10.json'
        """
        if format not in self.SIMPLE_FORMATS:
            raise ValueError('Unsupported file format')

        filename = 'toplist/%(count)d.%(format)s' % locals()
        return util.join(self._simple_base, filename)

    def suggestions_uri(self, count=10, format='opml'):
        """Get the Simple API URI for user suggestions

        >>> locator = Locator('john')
        >>> locator.suggestions_uri()
        'http://gpodder.net/suggestions/10.opml'
        >>> locator.suggestions_uri(50)
        'http://gpodder.net/suggestions/50.opml'
        >>> locator.suggestions_uri(70, 'json')
        'http://gpodder.net/suggestions/70.json'
        """
        if format not in self.SIMPLE_FORMATS:
            raise ValueError('Unsupported file format')

        filename = 'suggestions/%(count)d.%(format)s' % locals()
        return util.join(self._simple_base, filename)

    def search_uri(self, query, format='opml'):
        """Get the Simple API URI for podcast search

        >>> locator = Locator(None)
        >>> locator.search_uri('outlaws')
        'http://gpodder.net/search.opml?q=outlaws'
        >>> locator.search_uri(':something?', 'txt')
        'http://gpodder.net/search.txt?q=%3Asomething%3F'
        >>> locator.search_uri('software engineering', 'json')
        'http://gpodder.net/search.json?q=software+engineering'
        """
        if format not in self.SIMPLE_FORMATS:
            raise ValueError('Unsupported file format')

        query = urllib.quote_plus(query)
        filename = 'search.%(format)s?q=%(query)s' % locals()
        return util.join(self._simple_base, filename)

    def add_remove_subscriptions_uri(self, device_id):
        """Get the Advanced API URI for uploading list diffs

        >>> locator = Locator('bill')
        >>> locator.add_remove_subscriptions_uri('n810')
        'http://gpodder.net/api/2/subscriptions/bill/n810.json'
        """
        filename = '%(device_id)s.json' % locals()
        return util.join(self._base,
                'subscriptions', self._username, filename)

    def subscription_updates_uri(self, device_id, since=None):
        """Get the Advanced API URI for downloading list diffs

        The parameter "since" is optional and should be a numeric
        value (otherwise a ValueError is raised).

        >>> locator = Locator('jen')
        >>> locator.subscription_updates_uri('n900')
        'http://gpodder.net/api/2/subscriptions/jen/n900.json'
        >>> locator.subscription_updates_uri('n900', 1234)
        'http://gpodder.net/api/2/subscriptions/jen/n900.json?since=1234'
        """
        filename = '%(device_id)s.json' % locals()
        if since is not None:
            since = self._convert_since(since)
            filename += '?since=%(since)d' % locals()

        return util.join(self._base,
                'subscriptions', self._username, filename)

    def upload_episode_actions_uri(self):
        """Get the Advanced API URI for uploading episode actions

        >>> locator = Locator('thp')
        >>> locator.upload_episode_actions_uri()
        'http://gpodder.net/api/2/episodes/thp.json'
        """
        filename = self._username + '.json'
        return util.join(self._base, 'episodes', filename)

    def download_episode_actions_uri(self, since=None,
            podcast=None, device_id=None):
        """Get the Advanced API URI for downloading episode actions

        The parameter "since" is optional and should be a numeric
        value (otherwise a ValueError is raised).

        Both "podcast" and "device_id" are optional and exclusive:

        "podcast" should be a podcast URL
        "device_id" should be a device ID

        >>> locator = Locator('steve')
        >>> locator.download_episode_actions_uri()
        'http://gpodder.net/api/2/episodes/steve.json'
        >>> locator.download_episode_actions_uri(since=1337)
        'http://gpodder.net/api/2/episodes/steve.json?since=1337'
        >>> locator.download_episode_actions_uri(podcast='http://example.org/episodes.rss')
        'http://gpodder.net/api/2/episodes/steve.json?podcast=http%3A//example.org/episodes.rss'
        >>> locator.download_episode_actions_uri(since=2000, podcast='http://example.com/')
        'http://gpodder.net/api/2/episodes/steve.json?since=2000&podcast=http%3A//example.com/'
        >>> locator.download_episode_actions_uri(device_id='ipod')
        'http://gpodder.net/api/2/episodes/steve.json?device=ipod'
        >>> locator.download_episode_actions_uri(since=54321, device_id='ipod')
        'http://gpodder.net/api/2/episodes/steve.json?since=54321&device=ipod'
        """
        if podcast is not None and device_id is not None:
            raise ValueError('must not specify both "podcast" and "device_id"')

        filename = self._username + '.json'

        params = []
        if since is not None:
            since = str(self._convert_since(since))
            params.append(('since', since))

        if podcast is not None:
            params.append(('podcast', podcast))

        if device_id is not None:
            params.append(('device', device_id))

        if params:
            filename += '?' + '&'.join('%s=%s' % (key, urllib.quote(value)) for key, value in params)

        return util.join(self._base, 'episodes', filename)

    def device_settings_uri(self, device_id):
        """Get the Advanced API URI for setting per-device settings uploads

        >>> locator = Locator('mike')
        >>> locator.device_settings_uri('ipod')
        'http://gpodder.net/api/2/devices/mike/ipod.json'
        """
        filename = '%(device_id)s.json' % locals()
        return util.join(self._base, 'devices', self._username, filename)

    def device_list_uri(self):
        """Get the Advanced API URI for retrieving the device list

        >>> locator = Locator('jeff')
        >>> locator.device_list_uri()
        'http://gpodder.net/api/2/devices/jeff.json'
        """
        filename = self._username + '.json'
        return util.join(self._base, 'devices', filename)

    def toptags_uri(self, count=50):
        """Get the Advanced API URI for retrieving the top Tags

        >>> locator = Locator(None)
        >>> locator.toptags_uri()
        'http://gpodder.net/api/2/tags/50.json'
        >>> locator.toptags_uri(70)
        'http://gpodder.net/api/2/tags/70.json'
        """
        filename = '%(count)d.json' % locals()
        return util.join(self._base, 'tags', filename)

    def podcasts_of_a_tag_uri(self, tag, count=50):
        """Get the Advanced API URI for retrieving the top Podcasts of a Tag

        >>> locator = Locator(None)
        >>> locator.podcasts_of_a_tag_uri('linux')
        'http://gpodder.net/api/2/tag/linux/50.json'
        >>> locator.podcasts_of_a_tag_uri('linux',70)
        'http://gpodder.net/api/2/tag/linux/70.json'
        """
        filename = '%(tag)s/%(count)d.json' % locals()
        return util.join(self._base, 'tag', filename)

    def podcast_data_uri(self, podcast_url):
        """Get the Advanced API URI for retrieving Podcast Data

        >>> locator = Locator(None)
        >>> locator.podcast_data_uri('http://podcast.com')
        'http://gpodder.net/api/2/data/podcast.json?url=http%3A//podcast.com'
        """
        filename = 'podcast.json?url=%s' % urllib.quote(podcast_url) 
        return util.join(self._base, 'data', filename)
    
    def episode_data_uri(self, podcast_url, episode_url):
        """Get the Advanced API URI for retrieving Episode Data

        >>> locator = Locator(None)
        >>> locator.episode_data_uri('http://podcast.com','http://podcast.com/foo')
        'http://gpodder.net/api/2/data/episode.json?podcast=http%3A//podcast.com&url=http%3A//podcast.com/foo'
        """
        filename = 'episode.json?podcast=%s&url=%s' % (urllib.quote(podcast_url), urllib.quote(episode_url))
        return util.join(self._base, 'data', filename)

    def favorite_episodes_uri(self):
        """Get the Advanced API URI for listing favorite episodes

        >>> locator = Locator('mike')
        >>> locator.favorite_episodes_uri()
        'http://gpodder.net/api/2/favorites/mike.json'
        """
        filename = self._username + '.json'
        return util.join(self._base, 'favorites', filename)

    def settings_uri(self, type, scope_param1, scope_param2):
        """Get the Advanced API URI for retrieving or saving Settings
        
        Depending on the Type of setting scope_param2 or scope_param1 and scope_param2 are
        not necessary.

        >>> locator = Locator('joe')
        >>> locator.settings_uri('account',None,None)
        'http://gpodder.net/api/2/settings/joe/account.json'
        >>> locator.settings_uri('device','foodevice',None)
        'http://gpodder.net/api/2/settings/joe/device.json?device=foodevice'
        >>> locator.settings_uri('podcast','http://podcast.com',None)
        'http://gpodder.net/api/2/settings/joe/podcast.json?podcast=http%3A//podcast.com'
        >>> locator.settings_uri('episode','http://podcast.com','http://podcast.com/foo')
        'http://gpodder.net/api/2/settings/joe/episode.json?podcast=http%3A//podcast.com&episode=http%3A//podcast.com/foo'
        """
        if type not in self.SETTINGS_TYPES:
            raise ValueError('Unsupported Setting Type')

        filename = self._username + '/%(type)s.json' % locals()

        if type is 'device':
            if scope_param1 is None:
                raise ValueError('Devicename not specified')
            filename += '?device=%(scope_param1)s' % locals()

        if type is 'podcast':
            if scope_param1 is None:
                raise ValueError('Podcast URL not specified')
            filename += '?podcast=%s' % urllib.quote(scope_param1)

        if type is 'episode':
            if (scope_param1 is None) or (scope_param2 is None):
                raise ValueError('Podcast or Episode URL not specified')
            filename += '?podcast=%s&episode=%s' % (urllib.quote(scope_param1), urllib.quote(scope_param2))

        return util.join(self._base, 'settings' , filename)



# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mygpoclient

import datetime

def join(*args):
    """Join separate URL parts to a ful URL"""
    return '/'.join(args)

def iso8601_to_datetime(s):
    """Convert a ISO8601-formatted string to datetime

    >>> iso8601_to_datetime('2009-12-29T19:25:33')
    datetime.datetime(2009, 12, 29, 19, 25, 33)
    >>> iso8601_to_datetime('2009-12-29T19:25:33Z')
    datetime.datetime(2009, 12, 29, 19, 25, 33)
    >>> iso8601_to_datetime('xXxXxXxXxxxxXxxxXxx')
    >>>
    """
    for format in ('%Y-%m-%dT%H:%M:%S', '%Y-%m-%dT%H:%M:%SZ'):
        try:
            return datetime.datetime.strptime(s, format)
        except ValueError:
            continue

    return None

def datetime_to_iso8601(dt):
    """Convert a datetime to a ISO8601-formatted string

    >>> datetime_to_iso8601(datetime.datetime(2009, 12, 29, 19, 25, 33))
    '2009-12-29T19:25:33'
    """
    return dt.strftime('%Y-%m-%dT%H:%M:%S')

def position_to_seconds(s):
    """Convert a position string to its amount of seconds

    >>> position_to_seconds('00:00:01')
    1
    >>> position_to_seconds('00:01:00')
    60
    >>> position_to_seconds('01:00:00')
    3600
    >>> position_to_seconds('02:59:59')
    10799
    >>> position_to_seconds('100:00:00')
    360000
    """
    hours, minutes, seconds = (int(x) for x in s.split(':', 2))
    return (((hours*60)+minutes)*60)+seconds

def seconds_to_position(seconds):
    """Convert the amount of seconds to a position string

    >>> seconds_to_position(1)
    '00:00:01'
    >>> seconds_to_position(60)
    '00:01:00'
    >>> seconds_to_position(60*60)
    '01:00:00'
    >>> seconds_to_position(59 + 60*59 + 60*60*2)
    '02:59:59'
    >>> seconds_to_position(60*60*100)
    '100:00:00'
    """
    minutes = int(seconds/60)
    seconds = seconds % 60
    hours = int(minutes/60)
    minutes = minutes % 60
    return '%02d:%02d:%02d' % (hours, minutes, seconds)





# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import mygpoclient

from mygpoclient import locator
from mygpoclient import json
from mygpoclient import simple

class Tag(object):
    """Container class for a tag in the top tag list

    Attributes:
    tag - The name of the tag
    usage - Usage of the tag
    """

    REQUIRED_KEYS = ('tag', 'usage')

    def __init__(self, tag, usage):
        self.tag = tag
        self.usage = usage

    @classmethod
    def from_dict(cls, d):
        for key in cls.REQUIRED_KEYS:
            if key not in d:
                raise ValueError('Missing keys for tag')

        return cls(*(d.get(k) for k in cls.REQUIRED_KEYS))

    def __eq__(self, other):
        """Test two tag objects for equality

        >>> Tag('u', 123) == Tag('u', 123)
        True
        >>> Tag('u', 123) == Tag('a', 345)
        False
        >>> Tag('u', 123) == 'x'
        False
        """
        if not isinstance(other, self.__class__):
            return False

        return all(getattr(self, k) == getattr(other, k) \
                for k in self.REQUIRED_KEYS)

        

class Episode(object):
    """Container Class for Episodes
    
    Attributes:
    title -
    url -
    podcast_title -
    podcast_url -
    description -
    website -
    released -
    mygpo_link -
    """
    
    REQUIRED_KEYS = ('title', 'url', 'podcast_title', 'podcast_url',
                     'description', 'website', 'released', 'mygpo_link')
    
    def __init__(self, title, url, podcast_title, podcast_url, description, website, released, mygpo_link):
        self.title = title
        self.url = url
        self.podcast_title = podcast_title
        self.podcast_url = podcast_url
        self.description = description
        self.website = website
        self.released = released
        self.mygpo_link = mygpo_link
        
    @classmethod
    def from_dict(cls, d):
        for key in cls.REQUIRED_KEYS:
            if key not in d:
                raise ValueError('Missing keys for episode')
            
        return cls(*(d.get(k) for k in cls.REQUIRED_KEYS))
        
    def __eq__(self, other):
        """Test two Episode objects for equality
            
        >>> Episode('a','b','c','d','e','f','g','h') == Episode('a','b','c','d','e','f','g','h')
        True
        >>> Episode('a','b','c','d','e','f','g','h') == Episode('s','t','u','v','w','x','y','z')
        False
        >>> Episode('a','b','c','d','e','f','g','h') == 'x'
        False
        """
        if not isinstance(other, self.__class__):
            return False

        return all(getattr(self, k) == getattr(other, k) \
            for k in self.REQUIRED_KEYS)
                
class PublicClient(object):
    """Client for the gpodder.net "anonymous" API

    This is the API client implementation that provides a
    pythonic interface to the parts of the gpodder.net
    Simple API that don't need user authentication.
    """
    FORMAT = 'json'

    def __init__(self, host=mygpoclient.HOST, client_class=json.JsonClient):
        """Creates a new Public API client

        The parameter host is optional and defaults to
        the main webservice.

        The parameter client_class is optional and should
        not need to be changed in normal use cases. If it
        is changed, it should provide the same interface
        as the json.JsonClient class in mygpoclient.
        """
        self._locator = locator.Locator(None, host)
        self._client = client_class(None, None)

    def get_toplist(self, count=mygpoclient.TOPLIST_DEFAULT):
        """Get a list of most-subscribed podcasts

        Returns a list of simple.Podcast objects.

        The parameter "count" is optional and describes
        the amount of podcasts that are returned. The
        default value is 50, the minimum value is 1 and
        the maximum value is 100.
        """
        uri = self._locator.toplist_uri(count, self.FORMAT)
        return [simple.Podcast.from_dict(x) for x in self._client.GET(uri)]

    def search_podcasts(self, query):
        """Search for podcasts on the webservice

        Returns a list of simple.Podcast objects.

        The parameter "query" specifies the search
        query as a string.
        """
        uri = self._locator.search_uri(query, self.FORMAT)
        return [simple.Podcast.from_dict(x) for x in self._client.GET(uri)]

    def get_podcasts_of_a_tag(self, tag, count=mygpoclient.TOPLIST_DEFAULT):
        """Get a list of most-subscribed podcasts of a Tag

        Returns a list of simple.Podcast objects.

        The parameter "tag" specifies the tag as a String

        The parameter "count" is optional and describes
        the amount of podcasts that are returned. The
        default value is 50, the minimum value is 1 and
        the maximum value is 100.
        """
        uri = self._locator.podcasts_of_a_tag_uri(tag, count)
        return [simple.Podcast.from_dict(x) for x in self._client.GET(uri)]

    def get_toptags(self, count=mygpoclient.TOPLIST_DEFAULT):
        """Get a list of most-used tags

        Returns a list of Tag objects.

        The parameter "count" is optional and describes
        the amount of podcasts that are returned. The
        default value is 50, the minimum value is 1 and
        the maximum value is 100.
        """
        uri = self._locator.toptags_uri(count)
        return [Tag.from_dict(x) for x in self._client.GET(uri)]
    
    def get_podcast_data(self, podcast_uri):
        """Get Metadata for the specified Podcast
        
        Returns a simple.Podcast object.
        
        The parameter "podcast_uri" specifies the URL of the Podcast.
        """
        uri = self._locator.podcast_data_uri(podcast_uri)
        return simple.Podcast.from_dict(self._client.GET(uri))
    
    def get_episode_data(self, podcast_uri, episode_uri):
        """Get Metadata for the specified Episode
        
        Returns a Episode object.
        
        The parameter "podcast_uri" specifies the URL of the Podcast,
        which this Episode belongs to
        
        The parameter "episode_uri" specifies the URL of the Episode
        """
        uri = self._locator.episode_data_uri(podcast_uri, episode_uri)
        return Episode.from_dict(self._client.GET(uri))



# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mygpoclient import public
from mygpoclient import simple
from mygpoclient import testing

import unittest

class Test_Tag(unittest.TestCase):
    def test_tagFromDict_raisesValueError_missingKey(self):
        self.assertRaises(ValueError,public.Tag.from_dict, {'tag':'abcde'} )

class Test_Episode(unittest.TestCase):
    def test_episodeFromDict_raisesValueError_missingKey(self):
        self.assertRaises(ValueError,public.Episode.from_dict, {'title':'foobar','podcast_url':'http://www.podcast.com'})
        
class Test_PublicClient(unittest.TestCase):
    TOPLIST_JSON = """
    [{
    "website": "http://linuxoutlaws.com/podcast", 
    "description": "Open source talk with a serious attitude", 
    "title": "Linux Outlaws", 
    "url": "http://feeds.feedburner.com/linuxoutlaws", 
    "subscribers_last_week": 1736, 
    "subscribers": 1736, 
    "mygpo_link": "http://www.gpodder.net/podcast/11092", 
    "logo_url": "http://linuxoutlaws.com/files/albumart-itunes.jpg"
    }, 
    {
    "website": "http://syndication.mediafly.com/redirect/show/d581e9b773784df7a56f37e1138c037c", 
    "description": "We are not talking dentistry here; FLOSS all about Free Libre Open Source Software. Join hosts Randal Schwartz and Leo Laporte every Saturday as they talk with the most interesting and important people in the Open Source and Free Software community.", 
    "title": "FLOSS Weekly Video (large)", 
    "url": "http://feeds.twit.tv/floss_video_large",
    "subscribers_last_week": 50, 
    "subscribers": 50, 
    "mygpo_link": "http://www.gpodder.net/podcast/31991", 
    "logo_url": "http://static.mediafly.com/publisher/images/06cecab60c784f9d9866f5dcb73227c3/icon-150x150.png"
    }]
    """
    TOPLIST = [
            simple.Podcast('http://feeds.feedburner.com/linuxoutlaws',
                'Linux Outlaws',
                'Open source talk with a serious attitude',
                'http://linuxoutlaws.com/podcast',
                1736, 1736,
                'http://www.gpodder.net/podcast/11092',
                'http://linuxoutlaws.com/files/albumart-itunes.jpg'),
            simple.Podcast('http://feeds.twit.tv/floss_video_large',
                'FLOSS Weekly Video (large)',
                'We are not talking dentistry here; FLOSS all about Free Libre Open Source Software. Join hosts Randal Schwartz and Leo Laporte every Saturday as they talk with the most interesting and important people in the Open Source and Free Software community.',
                'http://syndication.mediafly.com/redirect/show/d581e9b773784df7a56f37e1138c037c',
                50, 50,
                'http://www.gpodder.net/podcast/31991',
                'http://static.mediafly.com/publisher/images/06cecab60c784f9d9866f5dcb73227c3/icon-150x150.png'),
    ]
    SEARCHRESULT_JSON = """
    [{
    "website": "http://linuxoutlaws.com/podcast", 
    "description": "Open source talk with a serious attitude", 
    "title": "Linux Outlaws", 
    "url": "http://feeds.feedburner.com/linuxoutlaws", 
    "subscribers_last_week": 1736, 
    "subscribers": 1736, 
    "mygpo_link": "http://www.gpodder.net/podcast/11092", 
    "logo_url": "http://linuxoutlaws.com/files/albumart-itunes.jpg"
    }, 
    {
    "website": "http://syndication.mediafly.com/redirect/show/d581e9b773784df7a56f37e1138c037c", 
    "description": "We are not talking dentistry here; FLOSS all about Free Libre Open Source Software. Join hosts Randal Schwartz and Leo Laporte every Saturday as they talk with the most interesting and important people in the Open Source and Free Software community.", 
    "title": "FLOSS Weekly Video (large)", 
    "url": "http://feeds.twit.tv/floss_video_large",
    "subscribers_last_week": 50, 
    "subscribers": 50, 
    "mygpo_link": "http://www.gpodder.net/podcast/31991", 
    "logo_url": "http://static.mediafly.com/publisher/images/06cecab60c784f9d9866f5dcb73227c3/icon-150x150.png"
    }]
    """
    SEARCHRESULT = [
            simple.Podcast('http://feeds.feedburner.com/linuxoutlaws',
                'Linux Outlaws',
                'Open source talk with a serious attitude',
                'http://linuxoutlaws.com/podcast',
                1736, 1736,
                'http://www.gpodder.net/podcast/11092',
                'http://linuxoutlaws.com/files/albumart-itunes.jpg'),
            simple.Podcast('http://feeds.twit.tv/floss_video_large',
                'FLOSS Weekly Video (large)',
                'We are not talking dentistry here; FLOSS all about Free Libre Open Source Software. Join hosts Randal Schwartz and Leo Laporte every Saturday as they talk with the most interesting and important people in the Open Source and Free Software community.',
                'http://syndication.mediafly.com/redirect/show/d581e9b773784df7a56f37e1138c037c',
                50, 50,
                'http://www.gpodder.net/podcast/31991',
                'http://static.mediafly.com/publisher/images/06cecab60c784f9d9866f5dcb73227c3/icon-150x150.png'),
    ]

    TOPTAGS_JSON = """
    [
        {"tag": "Technology",
         "usage": 530 },
        {"tag": "Arts",
         "usage": 400}
    ]
    """
    TOPTAGS = [
               public.Tag('Technology',530),
               public.Tag('Arts',400)
    ]
    
    PODCAST_JSON = """
    {
    "website": "http://linuxoutlaws.com/podcast", 
    "description": "Open source talk with a serious attitude", 
    "title": "Linux Outlaws", 
    "url": "http://feeds.feedburner.com/linuxoutlaws", 
    "subscribers_last_week": 1736, 
    "subscribers": 1736, 
    "mygpo_link": "http://www.gpodder.net/podcast/11092", 
    "logo_url": "http://linuxoutlaws.com/files/albumart-itunes.jpg"
    }
    """
    PODCAST = simple.Podcast('http://feeds.feedburner.com/linuxoutlaws',
                'Linux Outlaws',
                'Open source talk with a serious attitude',
                'http://linuxoutlaws.com/podcast',
                1736, 1736,
                'http://www.gpodder.net/podcast/11092',
                'http://linuxoutlaws.com/files/albumart-itunes.jpg')

    EPISODE_JSON = """
    {"title": "TWiT 245: No Hitler For You",
    "url": "http://www.podtrac.com/pts/redirect.mp3/aolradio.podcast.aol.com/twit/twit0245.mp3",
    "podcast_title": "this WEEK in TECH - MP3 Edition",
    "podcast_url": "http://leo.am/podcasts/twit", 
    "description": "[...]",
    "website": "http://www.podtrac.com/pts/redirect.mp3/aolradio.podcast.aol.com/twit/twit0245.mp3", 
    "released": "2010-12-25T00:30:00",
    "mygpo_link": "http://gpodder.net/episode/1046492"}
    """
    EPISODE = public.Episode('TWiT 245: No Hitler For You',
                             'http://www.podtrac.com/pts/redirect.mp3/aolradio.podcast.aol.com/twit/twit0245.mp3',
                             'this WEEK in TECH - MP3 Edition',
                             'http://leo.am/podcasts/twit',
                             '[...]',
                             'http://www.podtrac.com/pts/redirect.mp3/aolradio.podcast.aol.com/twit/twit0245.mp3',
                             '2010-12-25T00:30:00',
                             'http://gpodder.net/episode/1046492'
                             )

    def setUp(self):
        self.fake_client = testing.FakeJsonClient()
        self.client = public.PublicClient(client_class=self.fake_client)

    def test_getToplist(self):
        self.fake_client.response_value = self.TOPLIST_JSON
        result = self.client.get_toplist()
        self.assertEquals(result, self.TOPLIST)
        self.assertEquals(len(self.fake_client.requests), 1)

    def test_searchPodcasts(self):
        self.fake_client.response_value = self.SEARCHRESULT_JSON
        result = self.client.search_podcasts('wicked')
        self.assertEquals(result, self.SEARCHRESULT)
        self.assertEquals(len(self.fake_client.requests), 1)
        
    def test_getPodcastsOfATag(self):
        self.fake_client.response_value = self.SEARCHRESULT_JSON
        result = self.client.get_podcasts_of_a_tag('wicked')
        self.assertEquals(result, self.SEARCHRESULT)
        self.assertEquals(len(self.fake_client.requests), 1)

    def test_getTopTags(self):
        self.fake_client.response_value = self.TOPTAGS_JSON
        result = self.client.get_toptags()
        self.assertEquals(result, self.TOPTAGS)
        self.assertEquals(len(self.fake_client.requests), 1)
        
    def test_getPodcastData(self):
        self.fake_client.response_value = self.PODCAST_JSON
        result = self.client.get_podcast_data('http://feeds.feedburner.com/linuxoutlaws')
        self.assertEquals(result, self.PODCAST)
        self.assertEquals(len(self.fake_client.requests), 1)
        
    def test_getEpisodeData(self):
        self.fake_client.response_value = self.EPISODE_JSON
        result = self.client.get_episode_data('http://leo.am/podcasts/twit','http://www.podtrac.com/pts/redirect.mp3/aolradio.podcast.aol.com/twit/twit0245.mp3')
        self.assertEquals(result, self.EPISODE)
        self.assertEquals(len(self.fake_client.requests), 1)
        


# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from mygpoclient import simple
from mygpoclient import testing

import unittest

class Test_Podcast(unittest.TestCase):
    def test_podcastFromDict_raisesValueError_missingKey(self):
        self.assertRaises(ValueError,
                simple.Podcast.from_dict, {'url': 'a', 'title': 'b'})

class Test_SimpleClient(unittest.TestCase):
    USERNAME = 'a'
    PASSWORD = 'b'
    DEVICE_NAME = 'x'
    SUBSCRIPTIONS = [
            'http://lugradio.org/episodes.rss',
            'http://feeds2.feedburner.com/LinuxOutlaws',
    ]
    SUBSCRIPTIONS_JSON = """
      ["http://lugradio.org/episodes.rss",
       "http://feeds2.feedburner.com/LinuxOutlaws"]
    """
    SUGGESTIONS = [
        simple.Podcast('http://feeds.feedburner.com/linuxoutlaws',
                'Linux Outlaws',
                'Open source talk with a serious attitude',
                'http://linuxoutlaws.com/podcast',
                1736, 1736,
                'http://www.gpodder.net/podcast/11092',
                'http://linuxoutlaws.com/files/albumart-itunes.jpg'),
        simple.Podcast('http://feeds.twit.tv/floss_video_large',
                'FLOSS Weekly Video (large)',
                'We are not talking dentistry here; FLOSS all about Free Libre Open Source Software. Join hosts Randal Schwartz and Leo Laporte every Saturday as they talk with the most interesting and important people in the Open Source and Free Software community.',
                'http://syndication.mediafly.com/redirect/show/d581e9b773784df7a56f37e1138c037c',
                50, 50,
                'http://www.gpodder.net/podcast/31991',
                'http://static.mediafly.com/publisher/images/06cecab60c784f9d9866f5dcb73227c3/icon-150x150.png'),
    ]
    SUGGESTIONS_JSON = """
    [{
    "website": "http://linuxoutlaws.com/podcast", 
    "description": "Open source talk with a serious attitude", 
    "title": "Linux Outlaws", 
    "url": "http://feeds.feedburner.com/linuxoutlaws", 
    "subscribers_last_week": 1736, 
    "subscribers": 1736, 
    "mygpo_link": "http://www.gpodder.net/podcast/11092", 
    "logo_url": "http://linuxoutlaws.com/files/albumart-itunes.jpg"
    }, 
    {
    "website": "http://syndication.mediafly.com/redirect/show/d581e9b773784df7a56f37e1138c037c", 
    "description": "We are not talking dentistry here; FLOSS all about Free Libre Open Source Software. Join hosts Randal Schwartz and Leo Laporte every Saturday as they talk with the most interesting and important people in the Open Source and Free Software community.", 
    "title": "FLOSS Weekly Video (large)", 
    "url": "http://feeds.twit.tv/floss_video_large",
    "subscribers_last_week": 50, 
    "subscribers": 50, 
    "mygpo_link": "http://www.gpodder.net/podcast/31991", 
    "logo_url": "http://static.mediafly.com/publisher/images/06cecab60c784f9d9866f5dcb73227c3/icon-150x150.png"
    }]
    """

    def setUp(self):
        self.fake_client = testing.FakeJsonClient()
        self.client = simple.SimpleClient(self.USERNAME, self.PASSWORD,
                client_class=self.fake_client)

    def test_putSubscriptions(self):
        self.fake_client.response_value = ''
        result = self.client.put_subscriptions(self.DEVICE_NAME, self.SUBSCRIPTIONS)
        self.assertEquals(result, True)
        self.assertEquals(len(self.fake_client.requests), 1)

    def test_getSubscriptions(self):
        self.fake_client.response_value = self.SUBSCRIPTIONS_JSON
        subscriptions = self.client.get_subscriptions(self.DEVICE_NAME)
        self.assertEquals(subscriptions, self.SUBSCRIPTIONS)
        self.assertEquals(len(self.fake_client.requests), 1)

    def test_getSuggestions(self):
        self.fake_client.response_value = self.SUGGESTIONS_JSON
        suggestions = self.client.get_suggestions(50)
        self.assertEquals(suggestions, self.SUGGESTIONS)
        self.assertEquals(len(self.fake_client.requests), 1)


class Test_MissingCredentials(unittest.TestCase):
    DEVICE_NAME = 'unit-test-device'

    def test_getSubscriptions_UserAndPassAreNone(self):
        client = simple.SimpleClient(None, None, client_class=testing.FakeJsonClient())
        self.assertRaises(simple.MissingCredentials, client.get_subscriptions, (self.DEVICE_NAME,))

    def test_getSubscriptions_EmptyUserAndPass(self):
        client = simple.SimpleClient('', '', client_class=testing.FakeJsonClient())
        self.assertRaises(simple.MissingCredentials, client.get_subscriptions, (self.DEVICE_NAME,))

    def test_getSubscriptions_EmptyPassword(self):
        client = simple.SimpleClient('user', '', client_class=testing.FakeJsonClient())
        self.assertRaises(simple.MissingCredentials, client.get_subscriptions, (self.DEVICE_NAME,))

    def test_getSubscriptions_EmptyUsername(self):
        client = simple.SimpleClient('', 'pass', client_class=testing.FakeJsonClient())
        self.assertRaises(simple.MissingCredentials, client.get_subscriptions, (self.DEVICE_NAME,))




# -*- coding: utf-8 -*-
# gpodder.net API Client
# Copyright (C) 2009-2013 Thomas Perl and the gPodder Team
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from functools import wraps

import mygpoclient

from mygpoclient import locator
from mygpoclient import json


class MissingCredentials(Exception):
    """ Raised when instantiating a SimpleClient without credentials """


def needs_credentials(f):
    """ apply to all methods that initiate requests that require credentials """

    @wraps(f)
    def _wrapper(self, *args, **kwargs):
        if not self.username or not self.password:
            raise MissingCredentials

        return f(self, *args, **kwargs)

    return _wrapper



class Podcast(object):
    """Container class for a podcast

    Encapsulates the metadata for a podcast.

    Attributes:
    url - The URL of the podcast feed
    title - The title of the podcast
    description - The description of the podcast
    """
    REQUIRED_FIELDS = ('url', 'title', 'description', 'website', 'subscribers',
                       'subscribers_last_week', 'mygpo_link', 'logo_url')

    def __init__(self, url, title, description, website, subscribers, subscribers_last_week, mygpo_link, logo_url):
        self.url = url
        self.title = title
        self.description = description
        self.website = website
        self.subscribers = subscribers
        self.subscribers_last_week = subscribers_last_week
        self.mygpo_link = mygpo_link
        self.logo_url = logo_url

    @classmethod
    def from_dict(cls, d):
        for key in cls.REQUIRED_FIELDS:
            if key not in d:
                raise ValueError('Missing keys for toplist podcast')

        return cls(*(d.get(k) for k in cls.REQUIRED_FIELDS))

    def __eq__(self, other):
        """Test two Podcast objects for equality

        >>> Podcast('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h') == Podcast('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h')
        True
        >>> Podcast('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h') == Podcast('s', 't', 'u', 'v', 'w', 'x', 'y', 'z')
        False
        >>> Podcast('a', 'b', 'c', 'd', 'e', 'f', 'g', 'h') == 'a'
        False
        """
        if not isinstance(other, self.__class__):
            return False

        return all(getattr(self, k) == getattr(other, k) \
                for k in self.REQUIRED_FIELDS)


class SimpleClient(object):
    """Client for the gpodder.net Simple API

    This is the API client implementation that provides a
    pythonic interface to the gpodder.net Simple API.
    """
    FORMAT = 'json'

    def __init__(self, username, password, host=mygpoclient.HOST,
            client_class=json.JsonClient):
        """Creates a new Simple API client

        Username and password must be specified and are
        the user's login data for the webservice.

        The parameter host is optional and defaults to
        the main webservice.

        The parameter client_class is optional and should
        not need to be changed in normal use cases. If it
        is changed, it should provide the same interface
        as the json.JsonClient class in mygpoclient.
        """
        self.username = username
        self.password = password
        self._locator = locator.Locator(username, host)
        self._client = client_class(username, password)

    @needs_credentials
    def get_subscriptions(self, device_id):
        """Get a list of subscriptions for a device

        Returns a list of URLs (one per subscription) for
        the given device_id that reflects the current list
        of subscriptions.

        Raises http.NotFound if the device does not exist.
        """
        uri = self._locator.subscriptions_uri(device_id, self.FORMAT)
        return self._client.GET(uri)

    @needs_credentials
    def put_subscriptions(self, device_id, urls):
        """Update a device's subscription list

        Sets the server-side subscription list for the device
        "device_id" to be equivalent to the URLs in the list of
        strings "urls".

        The device will be created if it does not yet exist.

        Returns True if the update was successful, False otherwise.
        """
        uri = self._locator.subscriptions_uri(device_id, self.FORMAT)
        return (self._client.PUT(uri, urls) == None)

    @needs_credentials
    def get_suggestions(self, count=10):
        """Get podcast suggestions for the user

        Returns a list of Podcast objects that are
        to be suggested to the user.

        The parameter count is optional and if
        specified has to be a value between 1
        and 100 (with 10 being the default), and
        determines how much search results are
        returned (at maximum).
        """
        uri = self._locator.suggestions_uri(count, self.FORMAT)
        return [Podcast.from_dict(x) for x in self._client.GET(uri)]




#!/usr/bin/env python
# Generic setup script for single-package Python projects
# by Thomas Perl <thp.io/about>

from distutils.core import setup

import re
import os
import glob

PACKAGE = 'mygpoclient'
SCRIPT_FILE = os.path.join(PACKAGE, '__init__.py')

main_py = open(SCRIPT_FILE).read()
metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", main_py))
docstrings = re.findall('"""(.*?)"""', main_py, re.DOTALL)

# List the packages that need to be installed/packaged
PACKAGES = (
        PACKAGE,
)

SCRIPTS = glob.glob('scripts/*')

# Metadata fields extracted from SCRIPT_FILE
AUTHOR_EMAIL = metadata['author']
VERSION = metadata['version']
WEBSITE = metadata['website']
LICENSE = metadata['license']
DESCRIPTION = docstrings[0].strip()
if '\n\n' in DESCRIPTION:
    DESCRIPTION, LONG_DESCRIPTION = DESCRIPTION.split('\n\n', 1)
else:
    LONG_DESCRIPTION = None

# Extract name and e-mail ("Firstname Lastname <mail@example.org>")
AUTHOR, EMAIL = re.match(r'(.*) <(.*)>', AUTHOR_EMAIL).groups()

setup(name=PACKAGE,
      version=VERSION,
      description=DESCRIPTION,
      long_description=LONG_DESCRIPTION,
      author=AUTHOR,
      author_email=EMAIL,
      license=LICENSE,
      url=WEBSITE,
      packages=PACKAGES,
      scripts=SCRIPTS,
      download_url=WEBSITE+PACKAGE+'-'+VERSION+'.tar.gz')

